@@ -3,11 +3,14 @@
 from greynirseq.nicenlp.criterions import (
     multiclass_token_classification_criterion,
     multilabel_token_classification_criterion,
+    parser_criterion,
 )
 from greynirseq.nicenlp.data import datasets
-from greynirseq.nicenlp.models import multiclass, multilabel
+from greynirseq.nicenlp.models import multiclass, multilabel, simple_parser
 from greynirseq.nicenlp.tasks import (
     multiclass_token_classification_task,
     multilabel_token_classification_task,
+    parser_task,
     translation_with_backtranslation,
+    translation_with_glossary,
 )
